﻿using System;
using System.IO;
using System.Reflection;
using System.Windows.Media;
using Styx.Common;
using Styx.Plugins;
using System.Diagnostics;
using System.Text.RegularExpressions;
using System.Net;
using System.Collections.Generic;
using System.Linq;

namespace MeiamHelpers
{
    public class PluginLoader : HBPlugin
    {
        private HBPlugin _MeiamHelpers;

		private static bool _initialize;

        private const string SvnUrl = "http://code.taobao.org/svn/MeiamHelpers/trunk/";
        private const string VerUrl = "http://code.taobao.org/svn/MeiamHelpers/trunk/Changelog.txt";
        private static readonly string PluginPath = Utilities.AssemblyDirectory + @"\Plugins\" + "MeiamHelpers";
        private static string path = PluginPath + @"\MeiamHelpers.dll";

        private static readonly Regex _linkPattern = new Regex(@"<li><a href="".+"">(?<ln>.+(?:..))</a></li>",
                                                               RegexOptions.CultureInvariant);

        private PluginLoader()
        {
            if (_initialize)
                return;
            _initialize = true;

            CheckForUpdate();

            LoggingInfo("Load Plugin: " + path);

            try
            {
                byte[] Bytes = File.ReadAllBytes(path);
                Assembly asm = Assembly.Load(Bytes);

                foreach (Type t in asm.GetTypes())
                {
                    if (t.IsSubclassOf(typeof(HBPlugin)) && t.IsClass)
                    {
                        object obj = Activator.CreateInstance(t);
                        _MeiamHelpers = (HBPlugin)obj;
                    }
                }
            }
            catch (Exception error)
            {
                LoggingError(error.Message);
                LoggingInfo("版本加载失败.请登录 www.592.la 下载最新版本");
            }
        }

        private static void LoggingError(string message)
        {
            message = "[Loader] " + message;

            Trace.WriteLine(message);
            Logging.Write(Colors.OrangeRed, message);
        }

        private static void LoggingInfo(string message)
        {
            message = "[Loader] " + message;

            Trace.WriteLine(message);
            Logging.Write(Colors.DarkGray, message);
        }

        private static void LoggingWarning(string message)
        {
            message = "[Loader] " + message;

            Trace.WriteLine(message);
            Logging.Write(Colors.DarkOrange, message);
        }
        #region Overrides of MeiamHelpers

        public override string Name
        {
            get
            {
                return _MeiamHelpers.Name;
            }
        }
		
		public override string Author 
		{ 
			get 
			{ 
				return _MeiamHelpers.Author;
			} 
		}
		
		public override Version Version 
		{ 
			get 
			{ 
				return _MeiamHelpers.Version ;
			} 
		}

        public override string ButtonText
        {
            get
            {
                return _MeiamHelpers.ButtonText;
            }            
        }

        public override bool WantButton
        {
            get { return true; }
        }		

        public override void OnButtonPress()
        {
            _MeiamHelpers.OnButtonPress();
        }
				
		public override void OnDisable()
        {
            _MeiamHelpers.OnDisable();
        }
		
		public override void OnEnable()
        {
            _MeiamHelpers.OnEnable();
        }

		public override void Pulse()
        {
            _MeiamHelpers.Pulse();
        }

        #endregion

        #region 版本检测

        /// <summary>
        /// 获取文件的版本号
        /// </summary>
        /// <param name="filePath">文件的完整路径</param>
        /// <returns>文件的版本号</returns>
        private static string GetFileVersion(string filePath)
        {
            string FileVersions = "";

            try
            {
                FileVersionInfo file = FileVersionInfo.GetVersionInfo(filePath);
                FileVersions = file.ProductVersion;
            }
            catch (Exception)
            {
                FileVersions = "1.0.0";
            }

            return FileVersions;
        }

        public static void CheckForUpdate()
        {
            try
            {
                LoggingInfo("Checking for new version");
                Version remoteRev = GetRevision();
                if (Version.Parse(GetFileVersion(path)) < remoteRev)
                {
                    LoggingInfo(String.Format("A new version was found : {0}", remoteRev));
                    LoggingInfo("Downloading Update");
                    DownloadFilesFromSvn(new WebClient(), SvnUrl);
                    LoggingInfo("Download complete.");
                }
                else
                {
                    LoggingInfo("No updates found");
                }
            }
            catch (Exception ex)
            {
                LoggingInfo(ex.ToString());
                LoggingInfo("版本更新失败.请登录 www.592.la 下载最新版本");
            }
        }

        private static Version GetRevision()
        {
            var client = new WebClient();
            string html = client.DownloadString(VerUrl).Split(new char[] { '\n' })[0].Split(new char[] { ':' })[1];
            return Version.Parse(html);
            throw new Exception("Unable to retreive revision");
        }

        private static void DownloadFilesFromSvn(WebClient client, string url)
        {
            string html = client.DownloadString(url);
            MatchCollection results = _linkPattern.Matches(html);

            IEnumerable<Match> matches = from match in results.OfType<Match>()
                                         where match.Success && match.Groups["ln"].Success
                                         select match;
            foreach (Match match in matches)
            {
                string file = RemoveXmlEscapes(match.Groups["ln"].Value);
                string newUrl = url + file;
                if (newUrl[newUrl.Length - 1] == '/') // it's a directory...
                {
                    DownloadFilesFromSvn(client, newUrl);
                }
                else // its a file.
                {
                    string filePath, dirPath;
                    if (url.Length > SvnUrl.Length)
                    {
                        string relativePath = url.Substring(SvnUrl.Length);
                        dirPath = Path.Combine(PluginPath, relativePath);
                        filePath = Path.Combine(dirPath, file);
                    }
                    else
                    {
                        dirPath = Environment.CurrentDirectory;
                        filePath = Path.Combine(PluginPath, file);
                    }
                    LoggingInfo(String.Format("Downloading {0}", file));
                    if (!Directory.Exists(dirPath))
                        Directory.CreateDirectory(dirPath);
                    client.DownloadFile(newUrl, filePath);
                }
            }
        }

        private static string RemoveXmlEscapes(string xml)
        {
            return
                xml.Replace("&amp;", "&").Replace("&lt;", "<").Replace("&gt;", ">").Replace("&quot;", "\"").Replace(
                    "&apos;", "'");
        }
        #endregion
    }
}
